@value def: red;

.foo1 {
  prop: func(def);
}

.foo2 {
  prop: func(10px def);
}

.foo3 {
  prop: func(def 10px);
}

.foo4 {
  prop: func(10px def 10px);
}

.foo5 {
  prop: func(10px, def);
}

.foo6 {
  prop: func(def, 10px);
}

.foo7 {
  prop: func(10px, def, 10px);
}
