@import url(test.css);
@import url('test.css');
@import url("test.css");
@IMPORT url(test.css);
@import URL(test.css);
@import url(test.css );
@import url( test.css);
@import url( test.css );
@import url(
  test.css
);
@import url();
@import url('');
@import url("");
@import "test.css";
@import 'test.css';
@import '';
@import "";
@import "   ";
@import "
";
@import url();
@import url('');
@import url("");
@import url(test.css) screen and (orientation:landscape);
@import url(test.css) SCREEN AND (ORIENTATION: LANDSCAPE);
@import url(test.css)screen and (orientation:landscape);
@import url(test.css)  screen and (orientation:landscape);
@import url(test-media.css) screen and (orientation:landscape);
@import url(test-other.css) (min-width: 100px);
@import url(http://example.com/style.css);
@import url(http://example.com/style.css);
@import url(http://example.com/style.css#hash);
@import url(http://example.com/style.css?#hash);
@import url(http://example.com/style.css?foo=bar#hash);
@import url(http://example.com/other-style.css) screen and (orientation:landscape);
@import url(http://example.com/other-style.css) screen and (orientation:landscape);
@import url("//example.com/style.css");
@import url(~package/test.css);
@import ;
@import foo-bar;
@import-normalize;
@import url('http://') :root {}
@import url('query.css?foo=1&bar=1');
@import url('other-query.css?foo=1&bar=1#hash');
@import url('other-query.css?foo=1&bar=1#hash') screen and (orientation:landscape);
@import url('https://fonts.googleapis.com/css?family=Roboto');
@import url('https://fonts.googleapis.com/css?family=Noto+Sans+TC');
@import url('https://fonts.googleapis.com/css?family=Noto+Sans+TC|Roboto');

.class {
  a: b c d;
}

.foo {
  @import 'path.css';
}

@import url('./relative.css');
@import url('../import/top-relative.css');
@import url(~package/tilde.css);
@import url(~aliasesImport/alias.css);
@import url('./url.css');

.background {
  background: url('./img.png');
}

@import url(./test.css);

@import './te\
st.css';
@import './te\
\
\
st.css';
@import url('./te\
st.css');
@import url('./te\
\
\
st.css');

@import "./te'st.css";
@import url("./te'st.css");
@import './te\'st.css';
@import url('./te\'st.css');
@import './test test.css';
@import url('./test test.css');
@import './test\ test.css';
@import url('./test\ test.css');
@import './test%20test.css';
@import url('./test%20test.css');
@import './\74\65\73\74.css';
@import url('./\74\65\73\74.css');
@import './t\65\73\74.css';
@import url('./t\65\73\74.css');
@import url(./test\ test.css);
@import url(./t\65st%20test.css);
@import url('./t\65st%20test.css');
@import url("./t\65st%20test.css");
@import "./t\65st%20test.css";
@import './t\65st%20test.css';
@import url(                 test.css                      );
@import nourl(test.css);
@import '\
\
\
';
@import url('!!../../helpers/string-loader.js?esModule=false!~package/tilde.css');
@import url(test.css?foo=bar);
@import url(test.css?foo=bar#hash);
@import url(test.css?#hash);
@import "test.css" supports(display: flex);
@import "test.css" supports(display: flex) screen and (orientation:landscape);

/* Should be one import and two css modules */

@import url('something.css');
@import url('something.css');

/* Should be one import and two css modules */

@import url('something.css?foo=bar');
@import url('something.css?foo=bar');

/* Should be one import and two css modules */

@import url('something.css?foo=bar#hash');
@import url('something.css?foo=bar#hash');

/* Should be two import and two css modules */

@import url('something.css?foo=bar');
@import url('something.css?bar=foo');

/* Should be two import and two css modules */

@import url('something.css?foo=bar#one');
@import url('something.css?foo=bar#two');

/* Should be two import and two css modules */

@import url('something.css?foo=1&bar=2');
@import url('something.css?foo=2&bar=1');
