import { headerBaz, body, footer } from './index.css';

let html = '\n';

html += `<div class="${headerBaz}">\n`;
html += `<div class="${body}">\n`;
html += `<div class="${footer}">\n`;

__export__ =  html;

export default  html;
